@@ -19,6 +19,7 @@ import six
 import socket
 
 from openstack import _log
+from openstack import utils
 from openstack.cloud import exc
 
 
@@ -238,24 +239,36 @@ def find_best_address(addresses, public=False, cloud_public=True):
         # reachable. Otherwise we're just debug log spamming on every listing
         # of private ip addresses
         for address in addresses:
-            # Return the first one that is reachable
             try:
-                for res in socket.getaddrinfo(
-                        address, 22, socket.AF_UNSPEC, socket.SOCK_STREAM, 0):
-                    family, socktype, proto, _, sa = res
-                    connect_socket = socket.socket(family, socktype, proto)
-                    connect_socket.settimeout(1)
-                    connect_socket.connect(sa)
-                    return address
+                for count in utils.iterate_timeout(
+                        5, "Timeout waiting for %s" % address, wait=0.1):
+                    # Return the first one that is reachable
+                    try:
+                        for res in socket.getaddrinfo(
+                                address, 22, socket.AF_UNSPEC,
+                                socket.SOCK_STREAM, 0):
+                            family, socktype, proto, _, sa = res
+                            connect_socket = socket.socket(
+                                family, socktype, proto)
+                            connect_socket.settimeout(1)
+                            connect_socket.connect(sa)
+                            return address
+                    except socket.error:
+                        # Sometimes a "no route to address" type error
+                        # will fail fast, but can often come alive
+                        # when retried.
+                        continue
             except Exception:
                 pass
+
     # Give up and return the first - none work as far as we can tell
     if do_check:
         log = _log.setup_logging('openstack')
         log.debug(
-            'The cloud returned multiple addresses, and none of them seem'
-            ' to work. That might be what you wanted, but we have no clue'
-            " what's going on, so we just picked one at random")
+            "The cloud returned multiple addresses %s:, and we could not "
+            "connect to port 22 on either. That might be what you wanted, "
+            "but we have no clue what's going on, so we picked the first one "
+            "%s" % (addresses, addresses[0]))
     return addresses[0]
 
 
