@@ -10,8 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
-from openstack import resource2 as resource
+from openstack import resource
 from openstack import utils
 
 
@@ -19,15 +18,22 @@ class Project(resource.Resource):
     resource_key = 'project'
     resources_key = 'projects'
     base_path = '/projects'
-    service = identity_service.IdentityService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
-    patch_update = True
+    commit_method = 'PATCH'
+
+    _query_mapping = resource.QueryParameters(
+        'domain_id',
+        'is_domain',
+        'name',
+        'parent_id',
+        is_enabled='enabled',
+    )
 
     # Properties
     #: The description of the project. *Type: string*
@@ -56,7 +62,7 @@ class Project(resource.Resource):
         """Assign role to user on project"""
         url = utils.urljoin(self.base_path, self.id, 'users',
                             user.id, 'roles', role.id)
-        resp = session.put(url, endpoint_filter=self.service)
+        resp = session.put(url,)
         if resp.status_code == 204:
             return True
         return False
@@ -65,7 +71,7 @@ class Project(resource.Resource):
         """Validates that a user has a role on a project"""
         url = utils.urljoin(self.base_path, self.id, 'users',
                             user.id, 'roles', role.id)
-        resp = session.head(url, endpoint_filter=self.service)
+        resp = session.head(url,)
         if resp.status_code == 201:
             return True
         return False
@@ -74,7 +80,7 @@ class Project(resource.Resource):
         """Unassigns a role from a user on a project"""
         url = utils.urljoin(self.base_path, self.id, 'users',
                             user.id, 'roles', role.id)
-        resp = session.delete(url, endpoint_filter=self.service)
+        resp = session.delete(url,)
         if resp.status_code == 204:
             return True
         return False
@@ -83,7 +89,7 @@ class Project(resource.Resource):
         """Assign role to group on project"""
         url = utils.urljoin(self.base_path, self.id, 'groups',
                             group.id, 'roles', role.id)
-        resp = session.put(url, endpoint_filter=self.service)
+        resp = session.put(url,)
         if resp.status_code == 204:
             return True
         return False
@@ -92,7 +98,7 @@ class Project(resource.Resource):
         """Validates that a group has a role on a project"""
         url = utils.urljoin(self.base_path, self.id, 'groups',
                             group.id, 'roles', role.id)
-        resp = session.head(url, endpoint_filter=self.service)
+        resp = session.head(url,)
         if resp.status_code == 201:
             return True
         return False
@@ -101,7 +107,20 @@ class Project(resource.Resource):
         """Unassigns a role from a group on a project"""
         url = utils.urljoin(self.base_path, self.id, 'groups',
                             group.id, 'roles', role.id)
-        resp = session.delete(url, endpoint_filter=self.service)
+        resp = session.delete(url,)
         if resp.status_code == 204:
             return True
         return False
+
+
+class UserProject(Project):
+    resource_key = 'project'
+    resources_key = 'projects'
+    base_path = '/users/%(user_id)s/projects'
+
+    # capabilities
+    allow_create = False
+    allow_fetch = False
+    allow_commit = False
+    allow_delete = False
+    allow_list = True
