@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
 from openstack import resource
 
 
@@ -18,21 +17,18 @@ class Version(resource.Resource):
     resource_key = 'version'
     resources_key = 'versions'
     base_path = '/'
-    service = identity_service.IdentityService(
-        version=identity_service.IdentityService.UNVERSIONED
-    )
 
     # capabilities
     allow_list = True
 
     # Properties
-    media_types = resource.prop('media-types')
-    status = resource.prop('status')
-    updated = resource.prop('updated')
+    media_types = resource.Body('media-types')
+    status = resource.Body('status')
+    updated = resource.Body('updated')
 
     @classmethod
-    def list(cls, session, **params):
-        resp = session.get(cls.base_path, endpoint_filter=cls.service,
+    def list(cls, session, paginated=False, **params):
+        resp = session.get(cls.base_path,
                            params=params)
         resp = resp.json()
         for data in resp[cls.resources_key]['values']:
