@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
 from openstack.network.v2 import tag
 from openstack import resource
 
@@ -19,17 +18,18 @@ class Port(resource.Resource, tag.TagMixin):
     resource_key = 'port'
     resources_key = 'ports'
     base_path = '/ports'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
 
     # NOTE: we skip query on list or datetime fields for now
     _query_mapping = resource.QueryParameters(
+        'binding:host_id', 'binding:profile', 'binding:vif_details',
+        'binding:vif_type', 'binding:vnic_type',
         'description', 'device_id', 'device_owner', 'fixed_ips', 'ip_address',
         'mac_address', 'name', 'network_id', 'status', 'subnet_id',
         is_admin_state_up='admin_state_up',
