@@ -10,13 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import six
-
 from openstack import _adapter
-from openstack._meta import proxy as _meta
 from openstack import exceptions
 from openstack import resource
-from openstack import utils
 
 
 # The _check_resource decorator is used on Proxy methods to ensure that
@@ -43,9 +39,23 @@ def _check_resource(strict=False):
     return wrap
 
 
-class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
+class Proxy(_adapter.OpenStackSDKAdapter):
     """Represents a service."""
 
+    retriable_status_codes = None
+    """HTTP status codes that should be retried by default.
+
+    The number of retries is defined by the configuration in parameters called
+    ``<service-type>_status_code_retries``.
+    """
+
+    def __init__(self, *args, **kwargs):
+        # NOTE(dtantsur): keystoneauth defaults retriable_status_codes to None,
+        # override it with a class-level value.
+        kwargs.setdefault('retriable_status_codes',
+                          self.retriable_status_codes)
+        super(Proxy, self).__init__(*args, **kwargs)
+
     def _get_resource(self, resource_type, value, **attrs):
         """Get a resource object to work on
 
@@ -54,14 +64,18 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
                               :class:`~openstack.resource.Resource` with a
                               ``from_id`` method.
         :param value: The ID of a resource or an object of ``resource_type``
-                      class if using an existing instance, or None to create a
-                      new instance.
+                      class if using an existing instance, or ``munch.Munch``,
+                      or None to create a new instance.
         :param path_args: A dict containing arguments for forming the request
                           URL, if needed.
         """
         if value is None:
             # Create a bare resource
             res = resource_type.new(**attrs)
+        elif (isinstance(value, dict)
+              and not isinstance(value, resource.Resource)):
+            res = resource_type._from_munch(value)
+            res._update(**attrs)
         elif not isinstance(value, resource_type):
             # Create from an ID
             res = resource_type.new(id=value, **attrs)
@@ -138,16 +152,8 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
         res = self._get_resource(resource_type, value, **attrs)
 
         try:
-            rv = res.delete(
-                self,
-                error_message=(
-                    "Unable to delete {resource_type} for {value}".format(
-                        resource_type=resource_type.__name__,
-                        value=value,
-                    )
-                )
-            )
-        except exceptions.NotFoundException:
+            rv = res.delete(self)
+        except exceptions.ResourceNotFound:
             if ignore_missing:
                 return None
             raise
@@ -174,7 +180,7 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
         :rtype: :class:`~openstack.resource.Resource`
         """
         res = self._get_resource(resource_type, value, **attrs)
-        return res.update(self)
+        return res.commit(self)
 
     def _create(self, resource_type, **attrs):
         """Create a resource from attributes
@@ -198,7 +204,7 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
 
     @_check_resource(strict=False)
     def _get(self, resource_type, value=None, requires_id=True, **attrs):
-        """Get a resource
+        """Fetch a resource
 
         :param resource_type: The type of resource to get.
         :type resource_type: :class:`~openstack.resource.Resource`
@@ -212,12 +218,12 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
                            or :class:`~openstack.resource.Header`
                            values on this resource.
 
-        :returns: The result of the ``get``
+        :returns: The result of the ``fetch``
         :rtype: :class:`~openstack.resource.Resource`
         """
         res = self._get_resource(resource_type, value, **attrs)
 
-        return res.get(
+        return res.fetch(
             self, requires_id=requires_id,
             error_message="No {resource_type} found for {value}".format(
                 resource_type=resource_type.__name__, value=value))
@@ -267,61 +273,3 @@ class Proxy(six.with_metaclass(_meta.ProxyMeta, _adapter.OpenStackSDKAdapter)):
         """
         res = self._get_resource(resource_type, value, **attrs)
         return res.head(self)
-
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details=("This is no longer a part of the proxy base, "
-                               "service-specific subclasses should expose "
-                               "this as needed. See resource.wait_for_status "
-                               "for this behavior"))
-    def wait_for_status(self, value, status, failures=None, interval=2,
-                        wait=120):
-        """Wait for a resource to be in a particular status.
-
-        :param value: The resource to wait on to reach the status. The
-                      resource must have a status attribute.
-        :type value: :class:`~openstack.resource.Resource`
-        :param status: Desired status of the resource.
-        :param list failures: Statuses that would indicate the transition
-                              failed such as 'ERROR'.
-        :param interval: Number of seconds to wait between checks.
-        :param wait: Maximum number of seconds to wait for the change.
-
-        :return: Method returns resource on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
-                 to status failed to occur in wait seconds.
-        :raises: :class:`~openstack.exceptions.ResourceFailure` resource
-                 transitioned to one of the failure states.
-        :raises: :class:`~AttributeError` if the resource does not have a
-                 status attribute
-        """
-        failures = [] if failures is None else failures
-        return resource.wait_for_status(
-            self, value, status, failures, interval, wait)
-
-    @utils.deprecated(deprecated_in="0.9.14", removed_in="1.0",
-                      details=("This is no longer a part of the proxy base, "
-                               "service-specific subclasses should expose "
-                               "this as needed. See resource.wait_for_delete "
-                               "for this behavior"))
-    def wait_for_delete(self, value, interval=2, wait=120):
-        """Wait for the resource to be deleted.
-
-        :param value: The resource to wait on to be deleted.
-        :type value: :class:`~openstack.resource.Resource`
-        :param interval: Number of seconds to wait between checks.
-        :param wait: Maximum number of seconds to wait for the delete.
-
-        :return: Method returns resource on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
-                 to delete failed to occur in wait seconds.
-        """
-        return resource.wait_for_delete(self, value, interval, wait)
-
-
-class BaseProxy(Proxy):
-    # Backwards compat wrapper
-
-    @utils.deprecated(deprecated_in="0.11.1", removed_in="1.0",
-                      details="Use openstack.proxy.Proxy instead")
-    def __init__(self, *args, **kwargs):
-        super(BaseProxy, self).__init__(*args, **kwargs)
