@@ -34,15 +34,20 @@ and then returned to the caller.
 import collections
 import itertools
 
+import jsonpatch
 from keystoneauth1 import adapter
+from keystoneauth1 import discover
 import munch
 from requests import structures
+import six
 
 from openstack import _log
 from openstack import exceptions
 from openstack import format
 from openstack import utils
 
+_SEEN_FORMAT = '{name}_seen'
+
 
 def _convert_type(value, data_type, list_type=None):
     # This should allow handling list of dicts that have their own
@@ -83,35 +88,45 @@ class _BaseComponent(object):
     _map_cls = dict
 
     def __init__(self, name, type=None, default=None, alias=None,
-                 alternate_id=False, list_type=None, **kwargs):
+                 alternate_id=False, list_type=None, coerce_to_default=False,
+                 **kwargs):
         """A typed descriptor for a component that makes up a Resource
 
         :param name: The name this component exists as on the server
-        :param type: The type this component is expected to be by the server.
-                     By default this is None, meaning any value you specify
-                     will work. If you specify type=dict and then set a
-                     component to a string, __set__ will fail, for example.
+        :param type:
+            The type this component is expected to be by the server.
+            By default this is None, meaning any value you specify
+            will work. If you specify type=dict and then set a
+            component to a string, __set__ will fail, for example.
         :param default: Typically None, but any other default can be set.
         :param alias: If set, alternative attribute on object to return.
-        :param alternate_id: When `True`, this property is known
-                             internally as a value that can be sent
-                             with requests that require an ID but
-                             when `id` is not a name the Resource has.
-                             This is a relatively uncommon case, and this
-                             setting should only be used once per Resource.
-        :param list_type: If type is `list`, list_type designates what the
-                          type of the elements of the list should be.
+        :param alternate_id:
+            When `True`, this property is known internally as a value that
+            can be sent with requests that require an ID but when `id` is
+            not a name the Resource has. This is a relatively uncommon case,
+            and this setting should only be used once per Resource.
+        :param list_type:
+            If type is `list`, list_type designates what the type of the
+            elements of the list should be.
+        :param coerce_to_default:
+            If the Component is None or not present, force the given default
+            to be used. If a default is not given but a type is given,
+            construct an empty version of the type in question.
         """
         self.name = name
         self.type = type
-        self.default = default
+        if type is not None and coerce_to_default and not default:
+            self.default = type()
+        else:
+            self.default = default
         self.alias = alias
         self.alternate_id = alternate_id
         self.list_type = list_type
+        self.coerce_to_default = coerce_to_default
 
     def __get__(self, instance, owner):
         if instance is None:
-            return None
+            return self
 
         attributes = getattr(instance, self.key)
 
@@ -119,7 +134,25 @@ class _BaseComponent(object):
             value = attributes[self.name]
         except KeyError:
             if self.alias:
-                return getattr(instance, self.alias)
+                # Resource attributes can be aliased to each other. If neither
+                # of them exist, then simply doing a
+                # getattr(instance, self.alias) here sends things into
+                # infinite recursion (this _get method is what gets called
+                # when getattr(instance) is called.
+                # To combat that, we set a flag on the instance saying that
+                # we have seen the current name, and we check before trying
+                # to resolve the alias if there is already a flag set for that
+                # alias name. We then remove the seen flag for ourselves after
+                # we exit the alias getattr to clean up after ourselves for
+                # the next time.
+                alias_flag = _SEEN_FORMAT.format(name=self.alias)
+                if not getattr(instance, alias_flag, False):
+                    seen_flag = _SEEN_FORMAT.format(name=self.name)
+                    # Prevent infinite recursion
+                    setattr(instance, seen_flag, True)
+                    value = getattr(instance, self.alias)
+                    delattr(instance, seen_flag)
+                    return value
             return self.default
 
         # self.type() should not be called on None objects.
@@ -129,6 +162,8 @@ class _BaseComponent(object):
         return _convert_type(value, self.type, self.list_type)
 
     def __set__(self, instance, value):
+        if self.coerce_to_default and value is None:
+            value = self.default
         if value != self.default:
             value = _convert_type(value, self.type, self.list_type)
 
@@ -162,6 +197,12 @@ class URI(_BaseComponent):
     key = "_uri"
 
 
+class Computed(_BaseComponent):
+    """Computed attributes"""
+
+    key = "_computed"
+
+
 class _ComponentManager(collections.MutableMapping):
     """Storage of a component type"""
 
@@ -200,9 +241,16 @@ class _ComponentManager(collections.MutableMapping):
         return dict((key, self.attributes.get(key, None))
                     for key in self._dirty)
 
-    def clean(self):
-        """Signal that the resource no longer has modified attributes"""
-        self._dirty = set()
+    def clean(self, only=None):
+        """Signal that the resource no longer has modified attributes.
+
+        :param only: an optional set of attributes to no longer consider
+            changed
+        """
+        if only:
+            self._dirty = self._dirty - set(only)
+        else:
+            self._dirty = set()
 
 
 class _Request(object):
@@ -272,7 +320,16 @@ class QueryParameters(object):
         return result
 
 
-class Resource(object):
+class Resource(dict):
+    # TODO(mordred) While this behaves mostly like a munch for the purposes
+    # we need, sub-resources, such as Server.security_groups, which is a list
+    # of dicts, will contain lists of real dicts, not lists of munch-like dict
+    # objects. We should probably figure out a Resource class, perhaps
+    # SubResource or something, that we can use to define the data-model of
+    # complex object attributes where those attributes are not already covered
+    # by a different resource such as Server.image which should ultimately
+    # be an Image. We subclass dict so that things like json.dumps and pprint
+    # will work properly.
 
     #: Singular form of key for resource.
     resource_key = None
@@ -285,8 +342,8 @@ class Resource(object):
     id = Body("id")
     #: The name of this resource.
     name = Body("name")
-    #: The location of this resource.
-    location = Header("Location")
+    #: The OpenStack location of this resource.
+    location = Computed('location')
 
     #: Mapping of accepted query parameter names.
     _query_mapping = QueryParameters()
@@ -300,9 +357,9 @@ class Resource(object):
     #: Allow create operation for this resource.
     allow_create = False
     #: Allow get operation for this resource.
-    allow_get = False
+    allow_fetch = False
     #: Allow update operation for this resource.
-    allow_update = False
+    allow_commit = False
     #: Allow delete operation for this resource.
     allow_delete = False
     #: Allow list operation for this resource.
@@ -310,46 +367,97 @@ class Resource(object):
     #: Allow head operation for this resource.
     allow_head = False
 
-    #: Method for udating a resource (PUT, PATCH, POST)
-    update_method = "PUT"
+    # TODO(mordred) Unused - here for transition with OSC. Remove once
+    # OSC no longer checks for allow_get
+    allow_get = True
+
+    #: Method for committing a resource (PUT, PATCH, POST)
+    commit_method = "PUT"
     #: Method for creating a resource (POST, PUT)
     create_method = "POST"
+    #: Whether commit uses JSON patch format.
+    commit_jsonpatch = False
 
     #: Do calls for this resource require an id
     requires_id = True
     #: Do responses for this resource have bodies
     has_body = True
-    #: Is this a detailed version of another Resource
-    detail_for = None
 
-    def __init__(self, _synchronized=False, **attrs):
+    #: Maximum microversion to use for getting/creating/updating the Resource
+    _max_microversion = None
+    #: API microversion (string or None) this Resource was loaded with
+    microversion = None
+
+    _connection = None
+    _body = None
+    _header = None
+    _uri = None
+    _computed = None
+    _original_body = None
+
+    def __init__(self, _synchronized=False, connection=None, **attrs):
         """The base resource
 
-        :param bool _synchronized: This is not intended to be used directly.
-                    See :meth:`~openstack.resource.Resource.new` and
-                    :meth:`~openstack.resource.Resource.existing`.
+        :param bool _synchronized:
+            This is not intended to be used directly. See
+            :meth:`~openstack.resource.Resource.new` and
+            :meth:`~openstack.resource.Resource.existing`.
+        :param openstack.connection.Connection connection:
+            Reference to the Connection being used. Defaults to None to allow
+            Resource objects to be used without an active Connection, such as
+            in unit tests. Use of ``self._connection`` in Resource code should
+            protect itself with a check for None.
         """
-
+        self._connection = connection
+        self.microversion = attrs.pop('microversion', None)
         # NOTE: _collect_attrs modifies **attrs in place, removing
         # items as they match up with any of the body, header,
         # or uri mappings.
-        body, header, uri = self._collect_attrs(attrs)
+        body, header, uri, computed = self._collect_attrs(attrs)
         # TODO(briancurtin): at this point if attrs has anything left
         # they're not being set anywhere. Log this? Raise exception?
         # How strict should we be here? Should strict be an option?
 
-        self._body = _ComponentManager(attributes=body,
-                                       synchronized=_synchronized)
-        self._header = _ComponentManager(attributes=header,
-                                         synchronized=_synchronized)
-        self._uri = _ComponentManager(attributes=uri,
-                                      synchronized=_synchronized)
+        self._body = _ComponentManager(
+            attributes=body,
+            synchronized=_synchronized)
+        self._header = _ComponentManager(
+            attributes=header,
+            synchronized=_synchronized)
+        self._uri = _ComponentManager(
+            attributes=uri,
+            synchronized=_synchronized)
+        self._computed = _ComponentManager(
+            attributes=computed,
+            synchronized=_synchronized)
+        if self.commit_jsonpatch:
+            # We need the original body to compare against
+            if _synchronized:
+                self._original_body = self._body.attributes.copy()
+            elif self.id:
+                # Never record ID as dirty.
+                self._original_body = {
+                    self._alternate_id() or 'id': self.id
+                }
+            else:
+                self._original_body = {}
+
+        # TODO(mordred) This is terrible, but is a hack at the moment to ensure
+        # json.dumps works. The json library does basically if not obj: and
+        # obj.items() ... but I think the if not obj: is short-circuiting down
+        # in the C code and thus since we don't store the data in self[] it's
+        # always False even if we override __len__ or __bool__.
+        dict.update(self, self.to_dict())
 
     def __repr__(self):
-        pairs = ["%s=%s" % (k, v) for k, v in dict(itertools.chain(
-            self._body.attributes.items(),
-            self._header.attributes.items(),
-            self._uri.attributes.items())).items()]
+        pairs = [
+            "%s=%s" % (k, v if v is not None else 'None')
+            for k, v in dict(itertools.chain(
+                self._body.attributes.items(),
+                self._header.attributes.items(),
+                self._uri.attributes.items(),
+                self._computed.attributes.items())).items()
+        ]
         args = ", ".join(pairs)
 
         return "%s.%s(%s)" % (
@@ -359,9 +467,12 @@ class Resource(object):
         """Return True if another resource has the same contents"""
         if not isinstance(comparand, Resource):
             return False
-        return all([self._body.attributes == comparand._body.attributes,
-                    self._header.attributes == comparand._header.attributes,
-                    self._uri.attributes == comparand._uri.attributes])
+        return all([
+            self._body.attributes == comparand._body.attributes,
+            self._header.attributes == comparand._header.attributes,
+            self._uri.attributes == comparand._uri.attributes,
+            self._computed.attributes == comparand._computed.attributes,
+        ])
 
     def __getattribute__(self, name):
         """Return an attribute on this instance
@@ -381,6 +492,49 @@ class Resource(object):
         else:
             return object.__getattribute__(self, name)
 
+    def __getitem__(self, name):
+        """Provide dictionary access for elements of the data model."""
+        # Check the class, since BaseComponent is a descriptor and thus
+        # behaves like its wrapped content. If we get it on the class,
+        # it returns the BaseComponent itself, not the results of __get__.
+        real_item = getattr(self.__class__, name, None)
+        if isinstance(real_item, _BaseComponent):
+            return getattr(self, name)
+        raise KeyError(name)
+
+    def __delitem__(self, name):
+        delattr(self, name)
+
+    def __setitem__(self, name, value):
+        real_item = getattr(self.__class__, name, None)
+        if isinstance(real_item, _BaseComponent):
+            self.__setattr__(name, value)
+        else:
+            raise KeyError(
+                "{name} is not found. {module}.{cls} objects do not support"
+                " setting arbitrary keys through the"
+                " dict interface.".format(
+                    module=self.__module__,
+                    cls=self.__class__.__name__,
+                    name=name))
+
+    def keys(self):
+        # NOTE(mordred) In python2, dict.keys returns a list. In python3 it
+        # returns a dict_keys view. For 2, we can return a list from the
+        # itertools chain. In 3, return the chain so it's at least an iterator.
+        # It won't strictly speaking be an actual dict_keys, so it's possible
+        # we may want to get more clever, but for now let's see how far this
+        # will take us.
+        underlying_keys = itertools.chain(
+            self._body.attributes.keys(),
+            self._header.attributes.keys(),
+            self._uri.attributes.keys(),
+            self._computed.attributes.keys())
+        if six.PY2:
+            return list(underlying_keys)
+        else:
+            return underlying_keys
+
     def _update(self, **attrs):
         """Given attributes, update them on this instance
 
@@ -388,11 +542,20 @@ class Resource(object):
         layer when updating instances that may have already
         been created.
         """
-        body, header, uri = self._collect_attrs(attrs)
+        self.microversion = attrs.pop('microversion', None)
+        body, header, uri, computed = self._collect_attrs(attrs)
 
         self._body.update(body)
         self._header.update(header)
         self._uri.update(uri)
+        self._computed.update(computed)
+
+        # TODO(mordred) This is terrible, but is a hack at the moment to ensure
+        # json.dumps works. The json library does basically if not obj: and
+        # obj.items() ... but I think the if not obj: is short-circuiting down
+        # in the C code and thus since we don't store the data in self[] it's
+        # always False even if we override __len__ or __bool__.
+        dict.update(self, self.to_dict())
 
     def _collect_attrs(self, attrs):
         """Given attributes, return a dict per type of attribute
@@ -405,7 +568,25 @@ class Resource(object):
         header = self._consume_header_attrs(attrs)
         uri = self._consume_uri_attrs(attrs)
 
-        return body, header, uri
+        if any([body, header, uri]):
+            attrs = self._compute_attributes(body, header, uri)
+
+            body.update(self._consume_attrs(self._body_mapping(), attrs))
+
+            header.update(self._consume_attrs(self._header_mapping(), attrs))
+            uri.update(self._consume_attrs(self._uri_mapping(), attrs))
+        computed = self._consume_attrs(self._computed_mapping(), attrs)
+        # TODO(mordred) We should make a Location Resource and add it here
+        # instead of just the dict.
+        if self._connection:
+            computed['location'] = munch.unmunchify(
+                self._connection._openstackcloud.current_location)
+
+        return body, header, uri, computed
+
+    def _compute_attributes(self, body, header, uri):
+        """Compute additional attributes from the remote resource."""
+        return {}
 
     def _consume_body_attrs(self, attrs):
         return self._consume_mapped_attrs(Body, attrs)
@@ -500,6 +681,11 @@ class Resource(object):
         """Return all URI members of this class"""
         return cls._get_mapping(URI)
 
+    @classmethod
+    def _computed_mapping(cls):
+        """Return all URI members of this class"""
+        return cls._get_mapping(Computed)
+
     @classmethod
     def _alternate_id(cls):
         """Return the name of any value known as an alternate_id
@@ -546,7 +732,7 @@ class Resource(object):
         return cls(_synchronized=False, **kwargs)
 
     @classmethod
-    def existing(cls, **kwargs):
+    def existing(cls, connection=None, **kwargs):
         """Create an instance of an existing remote resource.
 
         When creating the instance set the ``_synchronized`` parameter
@@ -559,10 +745,14 @@ class Resource(object):
         :param dict kwargs: Each of the named arguments will be set as
                             attributes on the resulting Resource object.
         """
-        return cls(_synchronized=True, **kwargs)
+        res = cls(_synchronized=True, **kwargs)
+        # TODO(shade) Done as a second call rather than a constructor param
+        # because otherwise the mocking in the tests goes nuts.
+        res._connection = connection
+        return res
 
     @classmethod
-    def _from_munch(cls, obj, synchronized=True):
+    def _from_munch(cls, obj, synchronized=True, connection=None):
         """Create an instance from a ``munch.Munch`` object.
 
         This is intended as a temporary measure to convert between shade-style
@@ -572,16 +762,18 @@ class Resource(object):
         :param bool synchronized: whether this object already exists on server
             Must be set to ``False`` for newly created objects.
         """
-        return cls(_synchronized=synchronized, **obj)
+        return cls(_synchronized=synchronized, connection=connection, **obj)
 
-    def to_dict(self, body=True, headers=True, ignore_none=False,
-                original_names=False):
+    def to_dict(self, body=True, headers=True, computed=True,
+                ignore_none=False, original_names=False):
         """Return a dictionary of this resource's contents
 
         :param bool body: Include the :class:`~openstack.resource.Body`
                           attributes in the returned dictionary.
         :param bool headers: Include the :class:`~openstack.resource.Header`
                              attributes in the returned dictionary.
+        :param bool computed: Include the :class:`~openstack.resource.Computed`
+                              attributes in the returned dictionary.
         :param bool ignore_none: When True, exclude key/value pairs where
                                  the value is None. This will exclude
                                  attributes that the server hasn't returned.
@@ -598,9 +790,11 @@ class Resource(object):
             components.append(Body)
         if headers:
             components.append(Header)
+        if computed:
+            components.append(Computed)
         if not components:
             raise ValueError(
-                "At least one of `body` or `headers` must be True")
+                "At least one of `body`, `headers` or `computed` must be True")
 
         # isinstance stricly requires this to be a tuple
         components = tuple(components)
@@ -626,23 +820,40 @@ class Resource(object):
                             continue
                         if isinstance(value, Resource):
                             mapping[key] = value.to_dict()
-                        elif (value and isinstance(value, list)
-                              and isinstance(value[0], Resource)):
+                        elif value and isinstance(value, list):
                             converted = []
                             for raw in value:
-                                converted.append(raw.to_dict())
+                                if isinstance(raw, Resource):
+                                    converted.append(raw.to_dict())
+                                else:
+                                    converted.append(raw)
                             mapping[key] = converted
                         else:
                             mapping[key] = value
 
         return mapping
+    # Compatibility with the munch.Munch.toDict method
+    toDict = to_dict
+    # Make the munch copy method use to_dict
+    copy = to_dict
 
     def _to_munch(self):
         """Convert this resource into a Munch compatible with shade."""
         return munch.Munch(self.to_dict(body=True, headers=False,
                                         original_names=True))
 
-    def _prepare_request(self, requires_id=None, prepend_key=False):
+    def _prepare_request_body(self, patch, prepend_key):
+        if patch:
+            new = self._body.attributes
+            body = jsonpatch.make_patch(self._original_body, new).patch
+        else:
+            body = self._body.dirty
+            if prepend_key and self.resource_key is not None:
+                body = {self.resource_key: body}
+        return body
+
+    def _prepare_request(self, requires_id=None, prepend_key=False,
+                         patch=False):
         """Prepare a request to be sent to the server
 
         Create operations don't require an ID, but all others do,
@@ -651,6 +862,7 @@ class Resource(object):
         their bodies to be contained within an dict -- if the
         instance contains a resource_key and prepend_key=True,
         the body will be wrapped in a dict with that key.
+        If patch=True, a JSON patch is prepared instead of the full body.
 
         Return a _Request object that contains the constructed URI
         as well a body and headers that are ready to send.
@@ -659,9 +871,7 @@ class Resource(object):
         if requires_id is None:
             requires_id = self.requires_id
 
-        body = self._body.dirty
-        if prepend_key and self.resource_key is not None:
-            body = {self.resource_key: body}
+        body = self._prepare_request_body(patch, prepend_key)
 
         # TODO(mordred) Ensure headers have string values better than this
         headers = {}
@@ -701,6 +911,9 @@ class Resource(object):
             body = self._consume_body_attrs(body)
             self._body.attributes.update(body)
             self._body.clean()
+            if self.commit_jsonpatch:
+                # We need the original body to compare against
+                self._original_body = body.copy()
 
         headers = self._consume_header_attrs(response.headers)
         self._header.attributes.update(headers)
@@ -729,6 +942,80 @@ class Resource(object):
             " instance of an openstack.proxy.Proxy object or at the very least"
             " a raw keystoneauth1.adapter.Adapter.")
 
+    @classmethod
+    def _get_microversion_for_list(cls, session):
+        """Get microversion to use when listing resources.
+
+        The base version uses the following logic:
+        1. If the session has a default microversion for the current service,
+           just use it.
+        2. If ``self._max_microversion`` is not ``None``, use minimum between
+           it and the maximum microversion supported by the server.
+        3. Otherwise use ``None``.
+
+        Subclasses can override this method if more complex logic is needed.
+
+        :param session: :class`keystoneauth1.adapter.Adapter`
+        :return: microversion as string or ``None``
+        """
+        if session.default_microversion:
+            return session.default_microversion
+
+        return utils.maximum_supported_microversion(session,
+                                                    cls._max_microversion)
+
+    def _get_microversion_for(self, session, action):
+        """Get microversion to use for the given action.
+
+        The base version uses :meth:`_get_microversion_for_list`.
+        Subclasses can override this method if more complex logic is needed.
+
+        :param session: :class`keystoneauth1.adapter.Adapter`
+        :param action: One of "fetch", "commit", "create", "delete". Unused in
+            the base implementation.
+        :return: microversion as string or ``None``
+        """
+        if action not in ('fetch', 'commit', 'create', 'delete'):
+            raise ValueError('Invalid action: %s' % action)
+
+        return self._get_microversion_for_list(session)
+
+    def _assert_microversion_for(self, session, action, expected,
+                                 error_message=None):
+        """Enforce that the microversion for action satisfies the requirement.
+
+        :param session: :class`keystoneauth1.adapter.Adapter`
+        :param action: One of "fetch", "commit", "create", "delete".
+        :param expected: Expected microversion.
+        :param error_message: Optional error message with details. Will be
+            prepended to the message generated here.
+        :return: resulting microversion as string.
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the version
+            used for the action is lower than the expected one.
+        """
+        def _raise(message):
+            if error_message:
+                error_message.rstrip('.')
+                message = '%s. %s' % (error_message, message)
+
+            raise exceptions.NotSupported(message)
+
+        actual = self._get_microversion_for(session, action)
+        if actual is None:
+            message = ("API version %s is required, but the default "
+                       "version will be used.") % expected
+            _raise(message)
+
+        actual_n = discover.normalize_version_number(actual)
+        expected_n = discover.normalize_version_number(expected)
+        if actual_n < expected_n:
+            message = ("API version %(expected)s is required, but %(actual)s "
+                       "will be used.") % {'expected': expected,
+                                           'actual': actual}
+            _raise(message)
+
+        return actual
+
     def create(self, session, prepend_key=True):
         """Create a remote resource based on this instance.
 
@@ -746,24 +1033,28 @@ class Resource(object):
             raise exceptions.MethodNotSupported(self, "create")
 
         session = self._get_session(session)
+        microversion = self._get_microversion_for(session, 'create')
         if self.create_method == 'PUT':
             request = self._prepare_request(requires_id=True,
                                             prepend_key=prepend_key)
             response = session.put(request.url,
-                                   json=request.body, headers=request.headers)
+                                   json=request.body, headers=request.headers,
+                                   microversion=microversion)
         elif self.create_method == 'POST':
             request = self._prepare_request(requires_id=False,
                                             prepend_key=prepend_key)
             response = session.post(request.url,
-                                    json=request.body, headers=request.headers)
+                                    json=request.body, headers=request.headers,
+                                    microversion=microversion)
         else:
             raise exceptions.ResourceFailure(
                 msg="Invalid create method: %s" % self.create_method)
 
+        self.microversion = microversion
         self._translate_response(response)
         return self
 
-    def get(self, session, requires_id=True, error_message=None):
+    def fetch(self, session, requires_id=True, error_message=None):
         """Get a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -772,18 +1063,22 @@ class Resource(object):
                                     should be part of the requested URI.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
-                 :data:`Resource.allow_get` is not set to ``True``.
+                 :data:`Resource.allow_fetch` is not set to ``True``.
+        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
+                 the resource was not found.
         """
-        if not self.allow_get:
-            raise exceptions.MethodNotSupported(self, "get")
+        if not self.allow_fetch:
+            raise exceptions.MethodNotSupported(self, "fetch")
 
         request = self._prepare_request(requires_id=requires_id)
         session = self._get_session(session)
-        response = session.get(request.url)
+        microversion = self._get_microversion_for(session, 'fetch')
+        response = session.get(request.url, microversion=microversion)
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
 
+        self.microversion = microversion
         self._translate_response(response, **kwargs)
         return self
 
@@ -796,6 +1091,8 @@ class Resource(object):
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                  :data:`Resource.allow_head` is not set to ``True``.
+        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
+                 the resource was not found.
         """
         if not self.allow_head:
             raise exceptions.MethodNotSupported(self, "head")
@@ -803,51 +1100,79 @@ class Resource(object):
         request = self._prepare_request()
 
         session = self._get_session(session)
+        microversion = self._get_microversion_for(session, 'fetch')
         response = session.head(request.url,
-                                headers={"Accept": ""})
+                                headers={"Accept": ""},
+                                microversion=microversion)
 
+        self.microversion = microversion
         self._translate_response(response, has_body=False)
         return self
 
-    def update(self, session, prepend_key=True, has_body=True):
-        """Update the remote resource based on this instance.
+    def commit(self, session, prepend_key=True, has_body=True,
+               retry_on_conflict=None):
+        """Commit the state of the instance to the remote resource.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param prepend_key: A boolean indicating whether the resource_key
                             should be prepended in a resource update request.
                             Default to True.
+        :param bool retry_on_conflict: Whether to enable retries on HTTP
+                                       CONFLICT (409). Value of ``None`` leaves
+                                       the `Adapter` defaults.
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
-                 :data:`Resource.allow_update` is not set to ``True``.
+                 :data:`Resource.allow_commit` is not set to ``True``.
         """
-        # The id cannot be dirty for an update
+        # The id cannot be dirty for an commit
         self._body._dirty.discard("id")
 
-        # Only try to update if we actually have anything to update.
+        # Only try to update if we actually have anything to commit.
         if not any([self._body.dirty, self._header.dirty]):
             return self
 
-        if not self.allow_update:
-            raise exceptions.MethodNotSupported(self, "update")
+        if not self.allow_commit:
+            raise exceptions.MethodNotSupported(self, "commit")
 
-        request = self._prepare_request(prepend_key=prepend_key)
+        # Avoid providing patch unconditionally to avoid breaking subclasses
+        # without it.
+        kwargs = {}
+        if self.commit_jsonpatch:
+            kwargs['patch'] = True
+
+        request = self._prepare_request(prepend_key=prepend_key, **kwargs)
         session = self._get_session(session)
 
-        if self.update_method == 'PATCH':
+        kwargs = {}
+        retriable_status_codes = set(session.retriable_status_codes or ())
+        if retry_on_conflict:
+            kwargs['retriable_status_codes'] = retriable_status_codes | {409}
+        elif retry_on_conflict is not None and retriable_status_codes:
+            # The baremetal proxy defaults to retrying on conflict, allow
+            # overriding it via an explicit retry_on_conflict=False.
+            kwargs['retriable_status_codes'] = retriable_status_codes - {409}
+
+        microversion = self._get_microversion_for(session, 'commit')
+
+        if self.commit_method == 'PATCH':
             response = session.patch(
-                request.url, json=request.body, headers=request.headers)
-        elif self.update_method == 'POST':
+                request.url, json=request.body, headers=request.headers,
+                microversion=microversion, **kwargs)
+        elif self.commit_method == 'POST':
             response = session.post(
-                request.url, json=request.body, headers=request.headers)
-        elif self.update_method == 'PUT':
+                request.url, json=request.body, headers=request.headers,
+                microversion=microversion, **kwargs)
+        elif self.commit_method == 'PUT':
             response = session.put(
-                request.url, json=request.body, headers=request.headers)
+                request.url, json=request.body, headers=request.headers,
+                microversion=microversion, **kwargs)
         else:
             raise exceptions.ResourceFailure(
-                msg="Invalid update method: %s" % self.update_method)
+                msg="Invalid commit method: %s" % self.commit_method)
 
+        self.microversion = microversion
         self._translate_response(response, has_body=has_body)
         return self
 
@@ -859,16 +1184,20 @@ class Resource(object):
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
-                 :data:`Resource.allow_update` is not set to ``True``.
+                 :data:`Resource.allow_commit` is not set to ``True``.
+        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
+                 the resource was not found.
         """
         if not self.allow_delete:
             raise exceptions.MethodNotSupported(self, "delete")
 
         request = self._prepare_request()
         session = self._get_session(session)
+        microversion = self._get_microversion_for(session, 'delete')
 
         response = session.delete(request.url,
-                                  headers={"Accept": ""})
+                                  headers={"Accept": ""},
+                                  microversion=microversion)
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
@@ -910,6 +1239,7 @@ class Resource(object):
         if not cls.allow_list:
             raise exceptions.MethodNotSupported(cls, "list")
         session = cls._get_session(session)
+        microversion = cls._get_microversion_for_list(session)
 
         cls._query_mapping._validate(params, base_path=cls.base_path)
         query_params = cls._query_mapping._transpose(params)
@@ -925,7 +1255,8 @@ class Resource(object):
             response = session.get(
                 uri,
                 headers={"Accept": "application/json"},
-                params=query_params.copy())
+                params=query_params.copy(),
+                microversion=microversion)
             exceptions.raise_from_response(response)
             data = response.json()
 
@@ -950,7 +1281,11 @@ class Resource(object):
                 # argument and is practically a reserved word.
                 raw_resource.pop("self", None)
 
-                value = cls.existing(**raw_resource)
+                value = cls.existing(microversion=microversion, **raw_resource)
+                # TODO(shade) Done as a second call rather than a constructor
+                # param because otherwise the mocking in the tests goes nuts.
+                if hasattr(session, '_sdk_connection'):
+                    value._connection = session._sdk_connection
                 marker = value.id
                 yield value
                 total_yielded += 1
@@ -1058,7 +1393,11 @@ class Resource(object):
         # Try to short-circuit by looking directly for a matching ID.
         try:
             match = cls.existing(id=name_or_id, **params)
-            return match.get(session)
+            # TODO(shade) Done as a second call rather than a constructor
+            # param because otherwise the mocking in the tests goes nuts.
+            if hasattr(session, '_sdk_connection'):
+                match._connection = session._sdk_connection
+            return match.fetch(session)
         except exceptions.NotFoundException:
             pass
 
@@ -1124,7 +1463,7 @@ def wait_for_status(session, resource, status, failures, interval=None,
             timeout=wait,
             message=msg,
             wait=interval):
-        resource = resource.get(session)
+        resource = resource.fetch(session)
 
         if not resource:
             raise exceptions.ResourceFailure(
@@ -1166,7 +1505,7 @@ def wait_for_delete(session, resource, interval, wait):
                 id=resource.id),
             wait=interval):
         try:
-            resource = resource.get(session)
+            resource = resource.fetch(session)
             if not resource:
                 return orig_resource
             if resource.status.lower() == 'deleted':
