@@ -14,7 +14,7 @@
 
 import mock
 
-import openstack.cloud
+from openstack.cloud import openstackcloud
 from openstack.cloud import meta
 from openstack.tests import fakes
 from openstack.tests.unit import base
@@ -352,10 +352,10 @@ class TestMeta(base.TestCase):
             '10.0.0.101', meta.get_server_private_ip(srv, self.cloud))
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'has_service')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'has_service')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_private_ip_devstack(
             self,
             mock_get_flavor_name, mock_get_image_name,
@@ -417,9 +417,9 @@ class TestMeta(base.TestCase):
         self.assertEqual(PRIVATE_V4, srv['private_v4'])
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_private_ip_no_fip(
             self,
             mock_get_flavor_name, mock_get_image_name,
@@ -467,9 +467,9 @@ class TestMeta(base.TestCase):
         self.assertEqual(PRIVATE_V4, srv['private_v4'])
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_cloud_no_fips(
             self,
             mock_get_flavor_name, mock_get_image_name,
@@ -515,10 +515,10 @@ class TestMeta(base.TestCase):
         self.assertEqual(PRIVATE_V4, srv['private_v4'])
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'has_service')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'has_service')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_cloud_missing_fips(
             self,
             mock_get_flavor_name, mock_get_image_name,
@@ -584,9 +584,9 @@ class TestMeta(base.TestCase):
         self.assertEqual(PUBLIC_V4, srv['public_v4'])
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_cloud_rackspace_v6(
             self, mock_get_flavor_name, mock_get_image_name,
             mock_get_volumes):
@@ -634,9 +634,9 @@ class TestMeta(base.TestCase):
             "2001:4800:7819:103:be76:4eff:fe05:8525", srv['interface_ip'])
         self.assert_calls()
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_volumes')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_image_name')
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'get_flavor_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_volumes')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_image_name')
+    @mock.patch.object(openstackcloud._OpenStackCloudMixin, 'get_flavor_name')
     def test_get_server_cloud_osic_split(
             self, mock_get_flavor_name, mock_get_image_name,
             mock_get_volumes):
@@ -918,8 +918,8 @@ class TestMeta(base.TestCase):
         self.assertEqual('testgroup',
                          hostvars['security_groups'][0]['name'])
 
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv6')
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv4')
+    @mock.patch.object(meta, 'get_server_external_ipv6')
+    @mock.patch.object(meta, 'get_server_external_ipv4')
     def test_basic_hostvars(
             self, mock_get_server_external_ipv4,
             mock_get_server_external_ipv6):
@@ -952,8 +952,8 @@ class TestMeta(base.TestCase):
         # test volume exception
         self.assertEqual([], hostvars['volumes'])
 
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv6')
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv4')
+    @mock.patch.object(meta, 'get_server_external_ipv6')
+    @mock.patch.object(meta, 'get_server_external_ipv4')
     def test_ipv4_hostvars(
             self, mock_get_server_external_ipv4,
             mock_get_server_external_ipv6):
@@ -966,7 +966,7 @@ class TestMeta(base.TestCase):
             fake_cloud, meta.obj_to_munch(standard_fake_server))
         self.assertEqual(PUBLIC_V4, hostvars['interface_ip'])
 
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv4')
+    @mock.patch.object(meta, 'get_server_external_ipv4')
     def test_private_interface_ip(self, mock_get_server_external_ipv4):
         mock_get_server_external_ipv4.return_value = PUBLIC_V4
 
@@ -976,7 +976,7 @@ class TestMeta(base.TestCase):
             cloud, meta.obj_to_munch(standard_fake_server))
         self.assertEqual(PRIVATE_V4, hostvars['interface_ip'])
 
-    @mock.patch.object(openstack.cloud.meta, 'get_server_external_ipv4')
+    @mock.patch.object(meta, 'get_server_external_ipv4')
     def test_image_string(self, mock_get_server_external_ipv4):
         mock_get_server_external_ipv4.return_value = PUBLIC_V4
 
